@@ -77,6 +77,19 @@ impl Reader {
         Ok(json)
     }
 
+    pub fn from_manifest_data_and_stream(&self, manifest_data: &[u8], format: &str, stream: &dyn Stream) -> Result<String> {
+        // uniffi doesn't allow mutable parameters, so we we use an adapter
+        let mut stream = StreamAdapter::from(stream);
+        let reader = c2pa::Reader::from_manifest_data_and_stream(manifest_data, format, &mut stream)?;
+        let json = reader.to_string();
+        if let Ok(mut st) = self.reader.try_write() {
+            *st = reader;
+        } else {
+            return Err(Error::RwLock);
+        };
+        Ok(json)
+    }
+
     pub fn json(&self) -> Result<String> {
         if let Ok(st) = self.reader.try_read() {
             Ok(st.json())
@@ -121,6 +134,25 @@ impl Builder {
         Ok(())
     }
 
+    /// Set to true to disable embedding a manifest
+    pub fn set_no_embed(&self) -> Result<()> {
+        if let Ok(mut builder) = self.builder.try_write() {
+            builder.set_no_embed(true);
+        } else {
+            return Err(Error::RwLock);
+        };
+        Ok(())
+    }
+
+    pub fn set_remote_url(&self, remote_url: &str) -> Result<()> {
+        if let Ok(mut builder) = self.builder.try_write() {
+            builder.set_remote_url(remote_url);
+        } else {
+            return Err(Error::RwLock);
+        };
+        Ok(())
+    }
+
     /// Add a resource to the builder
     pub fn add_resource(&self, uri: &str, stream: &dyn Stream) -> Result<()> {
         if let Ok(mut builder) = self.builder.try_write() {
@@ -182,7 +214,7 @@ impl Builder {
         let mut dest = StreamAdapter::from(dest);
         if let Ok(mut builder) = self.builder.try_write() {
             let signer = (*signer).signer();
-            Ok(builder.sign(signer, format, &mut source, &mut dest)?)
+            Ok(builder.sign(signer.as_ref(), format, &mut source, &mut dest)?)
         } else {
             Err(Error::RwLock)
         }
@@ -192,7 +224,7 @@ impl Builder {
     pub fn sign_file(&self, signer: &CallbackSigner, source: &str, dest: &str) -> Result<Vec<u8>> {
         if let Ok(mut builder) = self.builder.try_write() {
             let signer = (*signer).signer();
-            Ok(builder.sign_file(signer, source, dest)?)
+            Ok(builder.sign_file(signer.as_ref(), source, dest)?)
         } else {
             Err(Error::RwLock)
         }
